package Foswiki::Plugins::RedmineQueryPlugin;
use strict;
use warnings;
use Getopt::Std;
use LWP::UserAgent;
use JSON;
use URI;
use IO::Socket::SSL;
use open ':locale';    # probe the locale environment variables like LANG

our $VERSION = '0.1';
our $RELEASE = '0.1';
our $SHORTDESCRIPTION =
  'A simple plugin to query Redmine issues over a REST API using JSON';
our $NO_PREFS_IN_TOPIC = 0;
our $pluginName        = 'RedmineQueryPlugin';

sub initPlugin {
    my ( $topic, $web, $user, $installWeb ) = @_;

    Foswiki::Func::registerTagHandler( 'RMQ', \&_RMQ );

    return 1;
}

sub _RMQ {
    my ( $session, $params, $theTopic, $theWeb ) = @_;

    my $default_format =
      Foswiki::Func::getPreferencesValue('REDMINEQUERYPLUGIN_FORMAT')
      || "| \$id | \$fixed_version | \$assigned_to | \$priority | \$status | \$tracker | \$subject |";
    my $default_tracker =
      Foswiki::Func::getPreferencesValue('REDMINEQUERYPLUGIN_TRACKER')
      || '1|2|3';
    my $default_status =
      Foswiki::Func::getPreferencesValue('REDMINEQUERYPLUGIN_STATUS')
      || '1|2|3|5|6';

    my $format  = $params->{'format'} || $default_format;
    my $status  = $params->{status}   || $default_status;
    my $tracker = $params->{tracker}  || $default_tracker;
    my $fixed_version = $params->{'fixed_version'};
    my $category      = $params->{'category'};
    my $id            = $params->{id};
    my $project_identifier =
      $Foswiki::cfg{Plugins}{RedmineQueryPlugin}{ProjectIdentifier} || '';
    my $limit = $Foswiki::cfg{Plugins}{RedmineQueryPlugin}{Limit} || 100;
    my $redmine_url =
      $Foswiki::cfg{Plugins}{RedmineQueryPlugin}{RedmineUrl} || '';
    my $api_key = $Foswiki::cfg{Plugins}{RedmineQueryPlugin}{ApiKey} || '';

    my $ua = new LWP::UserAgent;

    #$ua->timeout(10);    # default: 180sec
    $ua->default_header( 'X-Redmine-API-Key' => $api_key );

    my $project_url =
      URI->new("$redmine_url/projects/$project_identifier.json");
    my $project_res     = $ua->get($project_url);
    return 'Error getting project information' if $project_res->is_error;
    my $project_decoded = decode_json( $project_res->content );
    my $project_id      = $project_decoded->{'project'}{'id'};

        

    my %pars = (
        'status_id', $status, 'tracker_id', $tracker, 'project_id',
        $project_id, 'limit', $limit
    );

    if ($fixed_version) {
        my $versions_url =
          URI->new("$redmine_url/projects/$project_identifier/versions.json");
        my $versions_res     = $ua->get($versions_url);

	return 'Error getting versions for project' if $versions_res->is_error;
	
        my $versions_decoded = decode_json( $versions_res->content );
        my @versions         = @{ $versions_decoded->{'versions'} };
        foreach my $f (@versions) {
            if ( $f->{'name'} eq $fixed_version ) {
                $pars{'fixed_version_id'} = $f->{'id'};
            }
        }
        if ( !$pars{'fixed_version_id'} ) {
            return "Version unknown to Redmine";
        }
    }

    if ($category) {
        my $categories_url = URI->new(
            "$redmine_url/projects/$project_identifier/issue_categories.json");
        my $categories_res     = $ua->get($categories_url);

	return 'Error getting categories for project' if $categories_res->is_error;	

        my $categories_decoded = decode_json( $categories_res->content );
        my @categories         = @{ $categories_decoded->{'issue_categories'} };
        foreach my $f (@categories) {
            if ( $f->{'name'} eq $category ) {
                $pars{'category_id'} = $f->{'id'};
            }
        }
        if ( !$pars{'category_id'} ) { return "Category unknown to Redmine"; }
    }

    if ($id) {
        my $url = URI->new("$redmine_url/issues/$id.json");
  

        my $res = $ua->get($url);
        return "Error getting issue data" if $res->is_error;

        my $content = $res->content;

   #        $content = utf8::decode($content);    # convert UTF-8 binary to text

        my $content_decoded = decode_json($content);
        my $issue           = $content_decoded->{'issue'};


	if ($issue->{'project'}{'id'} eq $project_id) {
	    return getLine( $issue, $format );
	} else {
            return 'Issue not associated with this project';
	}
    }
    else {

        my $url = URI->new("$redmine_url/issues.json");

        $url->query_form(%pars);
        my $res = $ua->get($url);
        return 'Error getting issues data' if $res->is_error;

        my $content = $res->content;

       #    $content = utf8::decode($content);    # convert UTF-8 binary to text

        my $content_decoded = decode_json($content);

        my $retval = '';
        my @issues = @{ $content_decoded->{'issues'} };
        foreach my $f (@issues) {
            my $line = getLine( $f, $format );
            $retval .= $line . "\n";
        }
        return $retval;
    }

}

sub getLine {
    my ( $f, $format ) = @_;
    my $content_id            = $f->{'id'};
    my $content_subject       = $f->{'subject'};
    my $content_tracker       = $f->{'tracker'}{'name'};
    my $content_fixed_version = $f->{'fixed_version'}{'name'};
    my $content_assigned_to   = $f->{'assigned_to'}{'name'};
    my $content_priority      = $f->{'priority'}{'name'};
    my $content_status        = $f->{'status'}{'name'};
    my $content_category      = $f->{'category'}{'name'};
    my $content_due_date      = $f->{'due_date'};

    my $s = $format;
    $s =~ s/\$id/$content_id/;
    $s =~ s/\$subject/$content_subject/g;
    $s =~ s/\$tracker/$content_tracker/g;
    $s =~ s/\$fixed_version/$content_fixed_version/g;
    $s =~ s/\$assigned_to/$content_assigned_to/g;
    $s =~ s/\$priority/$content_priority/g;
    $s =~ s/\$status/$content_status/g;
    $s =~ s/\$due_date/$content_due_date/g;
    $s =~ s/\$category/$content_category/g;

    return $s;
}


